---
title: 'HOSR Core Rules: The Basics'
description: A basic overview of HOSR.
date_pub: 2023-04-26T22:53:00-04:00
section: rules
content_type: feature
short_code: uhb
---

Designed to emulate the fun and fantasy of the classic "Basic" and "Expert" editions of the world's most famous roleplaying system, without getting bogged down by an excess of rules.

These rules are still a Work-in-Progress, so if there's some rule or feature you need that isn't here yet, feel free to use the equivalent rule from your [favorite B/X-compatible retroclone](https://en.wikipedia.org/wiki/Dungeons_%26_Dragons_retro-clones) (we recommend [Old School Essentials](https://oldschoolessentials.necroticgnome.com/srd/index.php/Main_Page)).

[[toc]]

### How to Play

There is one _GameMaster_ (or _GM_). Everyone else is a _player_.

Each player controls a character (a _player-character_, or _PC_). All of the PCs together make up the _party_.

The GM's job is to describe what's happening in the game, and the players describe how their characters respond.

Sometimes, dice are rolled. These dice are described by how many sides they have (d4, d6, d8, d10, d12, d20, and d100).

### Doing Things

Simple tasks are assumed to succeed. If the task is dangerous or complicated, roll the dice for an [Ability Check](./adventuring.html#ability-checks).

For most checks, roll a 20-sided die (d20), add any modifiers, and compare the result with this table:

<div class="dividedTableWrapper">

|    Roll    |       Result        |
| :--------: | :-----------------: |
| Natural 20 |  Critical Success   |
|    11+     |  Complete Success   |
|    6-10    | Complicated Success |
| 5 or less  |       Failure       |
| Natural 1  |       Fumble        |

[Ability Check Result]

</div>

#### Advantage and Disadvantage

-   Rolling with **Advantage** means to roll 2 dice and keep the _highest_ result.
-   Rolling with **Disadvantage** means to roll 2 dice and keep the _lowest_ result.

### Abilities

Each PC has 6 abilities that help to define how they can interact with the world. Each ability has a score from 3 to 18 (to start), and each score determines the modifier used when rolling a check for that ability.

-   **Strength (STR)**: Raw physical power, affects melee damage and carrying capacity.
-   **Dexterity (DEX)**: Speed and agility of movement, affects _armor class (AC)_ when under attack.
-   **Constitution (CON)**: Mettle and gumption, affects _hit points (HP)_ gained at each level.
-   **Intelligence (INT)**: Knowledge and reasoning, affects languages known and arcane magic.
-   **Wisdom (WIS)**: Experience and insight, affects divine magic.
-   **Charisma (CHA)**: Appearance and social capability, affects hirelings and eldritch magic.

### Race, Class, and Level

Each PC has a race and levels in one or more classes.

-   A PC's race may influence their ability scores, and usually grants the PC one (or more) special features.
-   A PC's class determines what kind of weapons and armor they can use, how much damage they can take, and what special skills they have access to (including spellcasting).
    -   As a PC gains new levels in a class, they may gain additional special features.
    -   Some PCs can also multiclass, gaining levels in multiple different classes.

### Saving Throws

When something (bad) happens to a character or monster, they can usually make a saving throw to avoid some (or all) of it.

-   To make one, roll 1d20, add any appropriate modifiers, and compare the result with the **Save** value for your class and level.
-   Some types of saves are harder, and have different modifiers:
    -   **Save vs Wielded** rolls (avoiding wands, ray guns, or similar) get -1.
    -   **Save vs Blasts** rolls (avoiding explosions, breath weapons, or the like) get -3.
    -   **Save vs Spells** rolls (avoiding direct, targeted magic) get -4.

### Alignment

There are three alignments, cosmic principles that represent broad world views:

-   **Law**: Order, truth, justice, and civilization. Typically prosocial.
-   **Chaos**: Anarchy, power, chance, and destruction. Typically pro-individual.
-   **Neutrality**: Balance, mediation, and detachment.

### Character Creation

1. Roll Ability Scores.

    - Roll 4d6 and keep the highest 3 (aka "4d6KH3") for each ability.
    - Either roll 4d6KH3 once for each ability in order and see what kind of character comes up, or make all 6 rolls and assign the results to fit a particular class or concept.
    - On [Roll20](https://roll20.net/), all 6 scores can be rolled at once with this command:

```
/roll [[4d6kh3]] [[4d6kh3]] [[4d6kh3]] [[4d6kh3]] [[4d6kh3]] [[4d6kh3]]
```

2. Determine starting [Race](/races/) and [Class(es)](/classes/) (even when playing as a [human](/races/human.html)).
3. Determine starting level (ask the GM).
4. Note any special features the character has based on class, race, and level, including [class skills](./adventuring.html#skilled-ability-checks).
5. Roll Hit Dice to determine maximum [Hit Points](./adventuring.html#health-damage-and-dying).
    - Hit die type is determined by class, number of dice is determined by level.
    - Optionally, re-roll any 1s or 2s (once).
    - For each die, modify the rolled value by [CON modifier](./adventuring.html#constitution-modifier).
6. Choose [Alignment](#alignment).
7. Note starting [Languages](./adventuring.html#languages).
8. Determine starting Money.
    - For level 3 and under, begin with 3d6 x 10gp.
    - For level 4+, begin with (Level)d6 x 10gp.
9. Buy starting [Equipment](./equipment/fantasy.html).
10. Determine [Attacks and AC](./combat.html).

!!!include(license/wotc-cc-by.md)!!!
